from typing import *
from base_aux.funcs import IterAux, Explicit
from base_aux.classes.attr_3_dict_dots_1_simple import DictDots


# =====================================================================================================================
class DictDotsFull(DictDots):
    """
    TODO: do this perfect idea!

    smth like BS4 syntax but for dict/list
    1. use full access WR work with any level
    2. dict/list - ok! tuple - as generic final!
    3. keep source link! not just a simple copy (like in DictDots)!

    WHY NOT - DictDots
    ------------------
    it was a simple way to work with mainly first level! and only with dict!
    it seems it is obviously need use with lists same!
    """


# =====================================================================================================================
